from .sandbox import *  # noqa

MIGRATION_MODULES = {
    'address': None,
    'analytics': None,
    'basket': None,
    'reviews': None,
    'catalogue': None,
    'checkout': None,
    'customer': None,
    'dashboard': None,
    'offer': None,
    'order': None,
    'partner': None,
    'payment': None,
    'promotions': None,
    'search': None,
    'shipping': None,
    'voucher': None,
    'wishlists': None,
}
